indexing

	description	: "GERL implementation for ERL_ROUTINE"
	library: "Erl-G"
	copyright: "Copyright (c) 2005, Beat Fluri and others"
	license: "Eiffel Forum License v2 (see forum.txt)"
	date: "$Date$"
	revision: "$Revision$"

deferred class ERL_ROUTINE_IMP

inherit

	ERL_ROUTINE
		redefine
			precondition
		end

	ERL_FEATURE_IMP

feature -- Access

	precondition (a_target: ANY; a_arguments: TUPLE): BOOLEAN is
		do
			Result := precondition_function.item (operands (a_target, a_arguments))
		end

	formal_arguments: LIST [ERL_ARGUMENT]

feature -- Status Report

	valid_operands (a_target: ANY; a_arguments: TUPLE): BOOLEAN is
			-- Are `a_target' and `a_operands' valid operands?
		do
			Result := routine.valid_operands (operands (a_target, a_arguments))
		end

feature -- Basic Operations

	apply (a_target: ANY; a_arguments: TUPLE)is
			-- Call routine with operands `operands'.
		do
			routine.call (operands (a_target, a_arguments))
		end

feature {NONE} -- Implementation

	precondition_function: FUNCTION [ANY, TUPLE, BOOLEAN]
			-- Function representing precondition of `routine'

	routine: ROUTINE [ANY, TUPLE] is
			-- Routine
		deferred
		end

	operands (a_target: ANY; a_arguments: TUPLE): TUPLE is
			-- Tuple containing `a_target' and the items of `a_arguments'
		local
			i: INTEGER
		do
			Result := routine.operands.twin
			Result.put (a_target, 1)
			from
				i := 1
			until
				i > a_arguments.count
			loop
				Result.put (a_arguments.item (i), i + 1)
				i := i + 1
			end
		ensure
			operands_not_void: Result /= Void
			operands_count_valid: Result.count = a_arguments.count + 1
			target_set: Result.item (1) = a_target
		end

invariant

	precondition_function_not_void: precondition_function /= Void
	precondition_function_argument_match: precondition_function.operands.count = formal_arguments.count + 1
	routine_not_void: routine /= Void
	routine_argument_match: routine.operands.count = formal_arguments.count + 1

end
